/* 
 * 	    stegeek
 * 
 * simple data-existence-hiding tool
 * designed in 2005 and written in 2006 by niekt0 
 * (niekt0@hysteria.sk, https://hysteria.sk/~niekt0)
 * (please, mail me all discovered bugs/improvements.)
 *
 * 
 * version 0.9.9
 * 
 * newest version at http://stegeek.sf.net
 * 
 * Under GNU GPL.
 * See http://gnu.org for more information.
 * 
 * Use at you own risk.
 * 
 */


/* 2do:
 * 
 *	-- Computing of redundancy probability is only approximative. Potential problem, see documentation for more info.
 *	 
 * 	-- RAND_MAX (? problem or not?)
 * 	
 *	-- generating index:
 *		-- I am using 4 bytes from keygen as data for index. Is it safe? first 4, last 4, or from the middle?
 *		
 *	-- generate_junk:
 *		-- potential danger to use the same key in both cbc_enc and generating junk?  
 *		
 *	-- Optimize/test output file size
 *	
 *	-- Clean code:
 *		-- Check ALL possible key/data reusing/compromises
 *
 */

/* Limits of this version:
 *
 *  	-- max 2^32 = 4294967296 input files   ;)
 * 	-- max 16 TB size of archive (can be changed by using bigger BLOCK_SIZE)  
 *
 */


#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>
#include <string.h>
#include <sys/timeb.h>
#include <errno.h>
#include "aes/aes.h"

#define BLOCK_SIZE 4096   //4096

#define round(x) ((x)>=0?(int)((x)+0.5):(int)((x)-0.5))

struct onefile {	// information about file, we are adding

	int fd;			// file descriptor
	unsigned int id;	// id of file, used when adding file to archive
	unsigned long size;	// size of file
	char keygen[32];	// string to generate AES_KEY from
	AES_KEY key;		// AES_KEY structure
	char block[BLOCK_SIZE];	// one block of data

};

char ivec[BLOCK_SIZE]={0};	// we use ivec for AES_cbc mode in generat_junk too,
				// so it must be before main

char *errormsg;

void help(){
	
	printf("\nStegeek 0.9.9\n");
	printf("Creating archive: $ stegeek {-o output} -r redundancy input_files {< keyfile}\n");
	printf("(Note: -r 0.4 means 40%% redundancy, -r 1.5 means 150%% red...)\n");
	printf("Extracting from archive: $ stegeek {-o output} archive {< keyfile}\n\n");

}

void generate_junk(char *junkblock ,char *datagen, char *junkkeygen, AES_KEY *junkkey ,const int junksize){

int junkpom;

	// we will generate junksize of junk into junkblock 
	// encrypt each 32block into next 32block with junkkey
	// at the end, encrypt it all in cbc mode with same key, just to be sure about
	// statistic properties of final junk.
	// potential danger to use the same key? 

	memcpy(junkblock,datagen,32);
	
	AES_encrypt(junkblock,junkblock,junkkey);//BUGFIX
	
	for (junkpom=1; junkpom < junksize/32; junkpom++){
		AES_encrypt(junkblock+(junkpom-1)*32,junkblock+junkpom*32,junkkey );	
	}

	AES_encrypt(junkblock+(junkpom-1)*32,datagen,junkkey);

	AES_encrypt(junkkeygen,junkkeygen,junkkey);//BUGFIX
	AES_set_encrypt_key(junkkeygen,256,junkkey);

	memset(ivec,0,BLOCK_SIZE); // reset ivec !!! 
	AES_cbc_encrypt(junkblock,junkblock,junksize,junkkey,ivec,AES_ENCRYPT);
			
}



int main(int argc, char **argv)
{	
  	
	int out_file,rand_file;
	struct onefile *infiles;
	struct onefile extrfrom;
	char block[BLOCK_SIZE]= {0};

	struct stat sbuffer;
	int status=0,inputfiles=0,fixinputfiles;
	unsigned int outputsize=0;

	float redundancy = -1;

	AES_KEY randgen;
	unsigned long temprand[8],temp,temp2,index,maxred=0,curred=0;

	char temprand2[32];

	int i,j,mode=1;

	int opt;

	char helpoutputfile[20]="archive.sgk",helpextrfile[20]="extract.out";
	char helpchar[40] = {0};
	char *outputfile=helpoutputfile, *extrfile=helpextrfile, *randfile=NULL;

	struct timeb tp;

	// get_parameters

	static struct option long_opts[]={
		{"extract",0,0,'e'},    
		{"help",0,0,'h'},
		{"output",1,0,'o'},  
		{"randomsource",1,0,'s'},
		{"redundancy",1,0,'r'},
		{"version",0,0,'v'},
		{0,0,0,0}                             
	};



	while((opt = getopt_long(argc, argv, "eho:s:r:v",long_opts, NULL)) != -1) {
	  switch(opt) {
		case 'e': mode = 2; break;
		case 'h': help(); exit(0); break;
		case 'o': outputfile=optarg; extrfile=optarg; break;
		case 'r': redundancy=atof(optarg); break;
		case 's': randfile=optarg; break;	  
		case 'v': printf("Stegeek 0.9.9\n"); exit(0); break;
		case '?': fprintf(stderr, "Syntax error.\n"); help(); exit(7); break;	
	  }   	
	}	
       
	
	if (mode == 1) { //encrypting or decrypting
	
//-----------------------------------------------------------------------------------
//------------------------------------ Encrypting -----------------------------------
//-----------------------------------------------------------------------------------
	
//random initialization
	if ( randfile == NULL ) {	// if we get random from file or use system random
		
		ftime(&tp);
		srandom((int)tp.time);
		
	}	else {
		
		if ( (rand_file = open(randfile,O_RDONLY)) == -1 ) {
			errormsg = strerror(errno);
			fprintf(stderr, "%s : %s.\n",randfile,errormsg);
			exit(51);
		}	      
	}

// get_files	
	
	if ( redundancy < 0 ) { //bad or not specified redundancy
		printf("You HAVE TO specify redundancy, using default redundancy is pointless\n");
		help();
		exit(50);
	}

	for ( inputfiles = optind; argv[inputfiles] != NULL; inputfiles++ ) {
		printf("%s\n",argv[inputfiles]);
	}
	printf("-----\n");
	fixinputfiles = inputfiles-optind+1; // +1 because of redundancy

	infiles = calloc(fixinputfiles, sizeof( struct onefile));

	for ( inputfiles = 0; argv[optind] != NULL; optind++,inputfiles++ ) {
			
		if ( (infiles[inputfiles].fd = open(argv[optind],O_RDONLY)) == -1 ) {
			errormsg = strerror(errno);
			fprintf(stderr, "%s : %s.\n",argv[optind],errormsg);
			free(infiles);
			exit(1);
		}

		status = fstat(infiles[inputfiles].fd,&sbuffer); //get file size
		temp = sbuffer.st_size + 12 + (round(sbuffer.st_size/ BLOCK_SIZE)-1)*4; //add 12 bytes for id + 4 bytes for number of blocks
		infiles[inputfiles].size = sbuffer.st_size;
	
		if ( (temp % BLOCK_SIZE) != 0 ) { //round size to next block
			temp += (BLOCK_SIZE - (temp % BLOCK_SIZE)); // +BLOCK_SIZE debug
		}
	
		outputsize += temp;

	}
	
	if ( inputfiles == 0 ) { //if we don't have any input file
		help();
		free(infiles);
		exit(0);
	}
	
	printf("Adding %d files.\n",inputfiles);
	
	
// get_keys()

	printf("Enter encryption keys for files \n(10-36 chars, first 4 chars must differ from other keys of files in archive)\n");

	for ( i=0; i<inputfiles; i++ ) { // read keys for all input files 
		memset(helpchar,0,36);

		printf("key %d:",i);

		for ( j=0; j<36; j++ ) {  // read up to 36 chars
			/*(unsigned char)*/helpchar[j] = getchar();
		
			if ( helpchar[j] == '\n' ) { //end of line
				if (j < 5) {
					printf("Too short password, enter at least 5 chars ( better 36;)\n");
					printf("key %d:",i);
				 	j=-1; //start again 
				 } else {
					helpchar[j] = 0;
					break;
				}	
			}
		}
		
		if ( j == 36 ) { helpchar[38] = getchar(); }  //enter
		
		// if user gave us too short key...
		while ( strlen(helpchar) < 36 ) {
			j = strlen(helpchar);
			memcpy(helpchar+j,helpchar+4,((j-4)<(36-j))?(j-4):(36-j)); //i know about this feature, but it is OK.
		}	

		memcpy(&infiles[i].id,helpchar,4);
		memcpy(infiles[i].keygen,helpchar+4,32);

		AES_set_encrypt_key(infiles[i].keygen,256,&infiles[i].key);

	}

	printf("\nCreating archive...\n");

// compute output file size	

	maxred = (redundancy * outputsize) / BLOCK_SIZE;

	outputsize = round(outputsize * 1.4);  // to don't have too much collisions 

	outputsize = round( outputsize + redundancy*outputsize);

	if ( (outputsize % BLOCK_SIZE) != 0 ) { //dont need any more => WIPE
		outputsize += (BLOCK_SIZE - (outputsize % BLOCK_SIZE)); // +BLOCK_SIZE debug
	}
	
	if ( (out_file = open(outputfile,O_RDWR|O_CREAT,0666)) == -1 ) {
		errormsg = strerror(errno);
		fprintf(stderr, "%s : %s.\n",outputfile,errormsg);
		free(infiles);
		exit(4);
	}

	ftruncate(out_file,0);
	
	memset(block,0,BLOCK_SIZE); //create file containing only nulls
		
	for ( i=0; i<(outputsize/BLOCK_SIZE); i++ ) {
		if ( (status = write(out_file, block, BLOCK_SIZE)) == -1 ) {
			errormsg = strerror(errno);
			fprintf(stderr, "write() error: %s.\n",errormsg);
			free(infiles);
			exit(4);
		}
	}
	
	close(out_file); 

	if ( (out_file = open(outputfile,O_RDWR,0666)) == -1 ) {
		errormsg = strerror(errno);
		fprintf(stderr, "%s : %s.\n",outputfile,errormsg);
		free(infiles);
		exit(4);
	}

	// set up redundancy key (used only for filling empty space in last blocks of input files)
	
	if ( randfile == NULL ){
	
		for ( i=0; i<8; i++ ){
			temprand[i] = random();
		}

		for ( i=0; i<32; i+=4 ){
			temp = random();
			memcpy(temprand2+i,&temp,4);
			
		}

	}	else {
		
		if ( (status=read(rand_file,temprand,32)) == -1 ) {
			errormsg=strerror(errno);
			fprintf(stderr, "read() error in rand_file: %s.\n",errormsg);
			free(infiles);
			exit(53);
		}

		if ( (status=read(rand_file,temprand2,32)) == -1 ) {
			errormsg=strerror(errno);
			fprintf(stderr, "read() error in rand_file: %s.\n",errormsg);
			free(infiles);
			exit(53);
		}

		
		
	}	

	// generate 256bit=32byte "random" key
	memcpy(infiles[inputfiles].keygen,temprand,32);
	AES_set_encrypt_key(infiles[inputfiles].keygen,256,&infiles[inputfiles].key);

	
// Fill buffer with first blocks of files. Add file_size and id here

	for ( i=0; i<inputfiles; i++ ){
		memcpy(infiles[i].block,&infiles[i].id,4);
		memcpy(infiles[i].block+4,&infiles[i].size,8);

		if ( (status=read(infiles[i].fd,infiles[i].block+12,BLOCK_SIZE-12)) == -1 ){
			errormsg=strerror(errno);
			fprintf(stderr, "read() error in input file : %s.\n",errormsg);
			free(infiles);
			exit(5);
		
		}
		
		if ( status == 0 ) { 
			fprintf(stderr, "Unexpected end of file in input file, something is wrong..\n");
			free(infiles);
			exit(5);
			
		}	
	
		if ( status < BLOCK_SIZE-12 ) {
			//fill rest of block with random data
		 	generate_junk(infiles[inputfiles].block,temprand2,infiles[inputfiles].keygen,&infiles[inputfiles].key,BLOCK_SIZE-12-status); 
		}	
	}	
	
	for ( i=0; i<inputfiles; i++ ){ // get temp key, encrypt block, get another temp key

		memset(ivec,0,BLOCK_SIZE); // reset ivec !!! 
		AES_cbc_encrypt(infiles[i].block,infiles[i].block,BLOCK_SIZE, &infiles[i].key,ivec,AES_ENCRYPT); 

		AES_encrypt(infiles[i].keygen,infiles[i].keygen, &infiles[i].key);
		AES_set_encrypt_key(infiles[i].keygen,256,&infiles[i].key);
	}
	
	// set up redundancy key
	
	if ( randfile == NULL ){	
		for ( i=0; i<8; i++ ){
			temprand[i] = random();
		}
	}	else {
		
		if ( (status = read(rand_file,temprand,32)) == -1 ) {
			errormsg = strerror(errno);
			fprintf(stderr, "read() error in rand_file: %s.\n",errormsg);
			free(infiles);
			exit(54);
		}
	}
	

	// generate 256bit=32byte "random" key
	memcpy(infiles[inputfiles].keygen,temprand,32);
	AES_set_encrypt_key(infiles[inputfiles].keygen,256,&infiles[inputfiles].key);

 	generate_junk(infiles[inputfiles].block,temprand2,infiles[inputfiles].keygen,&infiles[inputfiles].key,BLOCK_SIZE); //fill redundancy block
 
	// set up random generator	
	if ( randfile == NULL ){	
		for ( i=0; i<8; i++ ){
			temprand[i] = random();
		}
	}	else {
		
		if ( (status = read(rand_file,temprand,32)) == -1 ) {
			errormsg = strerror(errno);
			fprintf(stderr, "read() error in rand_file: %s.\n",errormsg);
			free(infiles);
			exit(54);
		}
	}

	
	// generate 256bit=32byte "random" key
	AES_set_encrypt_key((char *)temprand,256,&randgen);


	// loop
	for ( ; inputfiles!=0; ) {
		//choose one of blocks

		memcpy(&temp,&temprand,4);
		
		if (curred == maxred ) {  // If we haven't add too much redundancy 
			temp = temp % (inputfiles);

		} else {
			temp = temp % (inputfiles+1+round(inputfiles*redundancy));
		}
		
			
		if (temp >= inputfiles) {
			curred++;
			temp = inputfiles;
		}	
		
		// get index in outfile(recursive)
		// now work only for archives <= 16 TB.. well,only;)
	
		
		temp2 = 1;
		
		for ( ; temp2!=0; ) {

		
			memcpy(&index,infiles[temp].keygen,4);  //was key before
			
			index = index % ((outputsize / BLOCK_SIZE));
	

			if ( (status = lseek(out_file,index*BLOCK_SIZE,SEEK_SET)) == -1 ) {
				errormsg = strerror(errno);
				fprintf(stderr, "lseek() error in out_file: %s.\n",errormsg);
				free(infiles);
				exit(6);
			}

			if ( (status = read(out_file,&temp2,4)) == -1 ) {
				errormsg = strerror(errno);
				fprintf(stderr, "read() error in out_file: %s.\n",errormsg);
				free(infiles);
				exit(7);
			}
			
			if ( status == 0 ) {
			       	fprintf(stderr, "Unexpected end of file in out_file. Something is wrong.\n"); 
				free(infiles);
				exit(7);
			}

			if ( (status=lseek(out_file,-4,SEEK_CUR)) == -1 ) {
				errormsg = strerror(errno);
				fprintf(stderr, "lseek() error in out_file: %s.\n",errormsg);
				free(infiles);
				exit(8);
			
			}
	
			AES_encrypt(infiles[temp].keygen,infiles[temp].keygen, &infiles[temp].key);
			AES_set_encrypt_key(infiles[temp].keygen,256,&infiles[temp].key);		
				
		}	
		
		// write block to outfile
		
		if ((status = write(out_file,infiles[temp].block,BLOCK_SIZE)) == -1 ) {
			errormsg = strerror(errno);
			fprintf(stderr, "write() error in out_file: %s.\n",errormsg);
			free(infiles);
			exit(9);
		}	
		// get another block instead of written one
		
		if (temp == inputfiles ){  // if it is redundancy
			
			// generate some junk 
			
			generate_junk(infiles[temp].block,temprand2,infiles[temp].keygen,&infiles[temp].key,BLOCK_SIZE);
			AES_encrypt(infiles[temp].keygen,infiles[temp].keygen, &infiles[temp].key);
			AES_set_encrypt_key(infiles[temp].keygen,256,&infiles[temp].key);		

		} else {  // it is one of files

			memcpy(infiles[temp].block,&infiles[temp].id,4); // write id first

			if ((status = read(infiles[temp].fd,infiles[temp].block+4,BLOCK_SIZE-4)) == -1 ) {
				errormsg = strerror(errno);
				fprintf(stderr, "read() error: %s.\n",errormsg);
				free(infiles);
				exit(10);
			}
			
			if (status != 0 ) { 
				// encrypt and generate another key
				memset(ivec,0,BLOCK_SIZE); // reset ivec !!! 
				AES_cbc_encrypt(infiles[temp].block,infiles[temp].block,BLOCK_SIZE, &infiles[temp].key,ivec,AES_ENCRYPT); 
				
				AES_encrypt(infiles[temp].keygen,infiles[temp].keygen, &infiles[temp].key);
				AES_set_encrypt_key(infiles[temp].keygen,256,&infiles[temp].key);		

			} else { // no more data in this file
				 // not needed any more, we will just set pointer pointing on him to st. else
				if (temp != inputfiles -1) { // if it is last file
					infiles[temp] = infiles[inputfiles-1]; // BUGFIX here !!!
				}
				infiles[inputfiles-1] = infiles[inputfiles];
				inputfiles--;
				
			}	
		
			if (status < BLOCK_SIZE-4) {
			//fill rest of block with random data
			 	generate_junk(infiles[inputfiles].block,temprand2,infiles[inputfiles].keygen,&infiles[inputfiles].key,BLOCK_SIZE-4-status); 
			}	

			
		}
		
		//generate another random key
		
		AES_encrypt((char *)temprand,(char *)temprand, (AES_KEY *)&randgen);
	}

	// fill rest of file with junk		
	
	if ( (status = lseek(out_file,0,SEEK_SET)) == -1 ) {
		errormsg = strerror(errno);
		fprintf(stderr, "lseek() error in out_file: %s.\n",errormsg);
		free(infiles);
		exit(11);
	}

	
	for (i=0;;i++){

			
		if ( (status = read(out_file,&temp2,4)) == -1 ) {
			errormsg = strerror(errno);
			fprintf(stderr, "read() error in out_file: %s.\n",errormsg);
			free(infiles);
			exit(11);
		}

		if ( status == 0 ) {
			//end of outfile
			close(out_file);
			free(infiles);
			printf("Done.\n");
			exit(0);
				
	       	}
	
		if ( temp2 == 0) {
			if ( (status=lseek(out_file,-4,SEEK_CUR)) == -1) {
				errormsg = strerror(errno);
				fprintf(stderr, "lseek() error in out_file: %s.\n",errormsg);
				free(infiles);
				exit(12);
			}
			
			generate_junk(infiles[0].block,temprand2,infiles[0].keygen,&infiles[0].key,BLOCK_SIZE);
			AES_encrypt(infiles[0].keygen,infiles[0].keygen, &infiles[0].key);
			AES_set_encrypt_key(infiles[0].keygen,256,&infiles[0].key);		
									
			if ( (status = write(out_file, infiles[0].block, BLOCK_SIZE)) == -1 ){
				errormsg = strerror(errno);
				fprintf(stderr, "write() error in out_file: %s.\n",errormsg);
				free(infiles);
				exit(13);
			}
			
		} else {

			if ( (status = read(out_file,infiles[0].block,BLOCK_SIZE-4)) == -1 ) {
				errormsg = strerror(errno);
				fprintf(stderr, "read() error in out_file: %s.\n",errormsg);
				free(infiles);
				exit(12);
			}
		}
	}

	//close output
	close(out_file);	
	free(infiles);
	printf("Done.\n");	
	exit(0);
	
//----------------------------------------------------------------------------------------------
//---------------------------------------- Extracting ------------------------------------------
//----------------------------------------------------------------------------------------------
	
 } else {    //extract from archive

	if ( randfile != NULL ){
		printf("Cannot specify --randomsource and --extract at once\n");
		help();
		exit(55);
	}       
	
	if ( redundancy >= 0 ){
		printf("Cannot specify --redundancy and --extract at once\n");
		help();
		exit(56);

	}	
	
	if ( argv[optind]!=NULL ) {
		if (argv[optind+1] != NULL) {
			fprintf(stderr, "Syntax error.\n"); 
			exit(7);
		} else
		{
			outputfile = argv[optind];
		}

	}

	if ( (extrfrom.fd = open(outputfile,O_RDONLY)) == -1 ) {
		errormsg = strerror(errno);
		fprintf(stderr, "%s: %s.\n",outputfile,errormsg);
		exit(21);
	}

	//set extraction key (&id)

	memset(helpchar,0,36);

	printf("Enter extraction key: ");

	for ( j=0; j<36; j++ ) {
		helpchar[j] = getchar();
		if (helpchar[j] == '\n') {
			helpchar[j] = 0;
			break;
		}
	}

	if ( j==36 ) { helpchar[38] = getchar(); }  //enter
	
	printf("Extracting...\n");	

	// if user gave us too short key...
	while ( strlen(helpchar) < 36 ) {
		j = strlen(helpchar);
		memcpy(helpchar+j,helpchar+4,((j-4)<(36-j))?(j-4):(36-j)); //i know about this feature, but it is OK.
	}
	
	memcpy(&extrfrom.id,helpchar,4);
	memcpy(extrfrom.keygen,helpchar+4,32);
	
	//create our output file
	
	if ( (out_file = open(extrfile,O_RDWR|O_CREAT,0666)) == -1 ) {
		errormsg = strerror(errno);
		fprintf(stderr, "%s: %s.\n",outputfile,errormsg);
		exit(22);
	}

	ftruncate(out_file,0);


	status = fstat(extrfrom.fd,&sbuffer);
	extrfrom.size=sbuffer.st_size;

	AES_set_encrypt_key(extrfrom.keygen,256,&extrfrom.key);		
	AES_set_decrypt_key(extrfrom.keygen,256,&randgen);	
	
	AES_encrypt(extrfrom.keygen,extrfrom.keygen, &extrfrom.key);
	AES_set_encrypt_key(extrfrom.keygen,256,&extrfrom.key);	

	//index in file:
	
	for ( j=0,temp2=1; ((temp2!=0)&&(j < extrfrom.size/BLOCK_SIZE)); j++ ) {

		memcpy(&index,extrfrom.keygen,4);  //was key before
		
		index = index % ((extrfrom.size / BLOCK_SIZE));

		if ( (status = lseek(extrfrom.fd,index*BLOCK_SIZE,SEEK_SET)) == -1 ) {
			errormsg = strerror(errno);
			fprintf(stderr, "lseek() error in in_file: %s.\n",errormsg);
			exit(23);
		}

		if ( (status = read(extrfrom.fd,extrfrom.block,BLOCK_SIZE)) == -1 ) {
			errormsg = strerror(errno);
			fprintf(stderr, "read() error in in_file: %s.\n",errormsg);
			exit(24);
		}

		if (status == 0 ) { 
			fprintf(stderr, "End of file. Something is wrong.(filesystem?)\n"); 
			exit(25);

		}

		//decrypt block
		memset(ivec,0,BLOCK_SIZE); // reset ivec !!! 
		AES_cbc_encrypt(extrfrom.block,extrfrom.block,BLOCK_SIZE,&randgen,ivec,AES_DECRYPT); 

		memcpy(&temp2,extrfrom.block,4);
		
		if ( temp2 == extrfrom.id ) {  // if we have found our first block
			memcpy(&outputsize,extrfrom.block+4,8);

			temp2 = 0;
		
			AES_encrypt(extrfrom.keygen,extrfrom.keygen, &extrfrom.key);

			AES_set_decrypt_key(extrfrom.keygen,256,&randgen);	
	
			AES_set_encrypt_key(extrfrom.keygen,256,&extrfrom.key);

	
		} else {
		       	temp2 = 1; 
		}
		
		// generate next index
		
		AES_encrypt(extrfrom.keygen,extrfrom.keygen, &extrfrom.key);

		AES_set_encrypt_key(extrfrom.keygen,256,&extrfrom.key);
				
	}	
		
	if ( !(j < extrfrom.size/BLOCK_SIZE) ) {
		printf("Probably bad key?\n");
		exit(0);
	}

	// write our block to outfile
	if ( outputsize < (BLOCK_SIZE-12) ) { 	
		if ( (status = write(out_file, extrfrom.block+12, outputsize)) == -1 ){
			errormsg = strerror(errno);
			fprintf(stderr, "write() error in out_file: %s.\n",errormsg);
			exit(26);
		}
		
		close(out_file);	
		printf("Done.\n");
		exit(0);
		
	} else
	{
		if ( (status = write(out_file, extrfrom.block+12, BLOCK_SIZE-12)) == -1 ) {
			errormsg = strerror(errno);
			fprintf(stderr, "write() error in out_file: %s.\n",errormsg);
			exit(27);
		}

	}

	outputsize -= BLOCK_SIZE-12;
	
	for ( j=0; (j < extrfrom.size/BLOCK_SIZE); j++ ) {

		memcpy(&index,&extrfrom.keygen,4);
		index = index % ((extrfrom.size / BLOCK_SIZE));

		if ( (status = lseek(extrfrom.fd,index*BLOCK_SIZE,SEEK_SET)) == -1 ) {
			errormsg = strerror(errno);
			fprintf(stderr, "lseek() error in in_file: %s.\n",errormsg);
			exit(28);
		}

		if ( (status = read(extrfrom.fd,extrfrom.block,BLOCK_SIZE)) == -1 ) {
			errormsg = strerror(errno);
			fprintf(stderr, "read() error in in_file: %s.\n",errormsg);
			exit(29);
		}

		if ( status == 0 ) { 
			fprintf(stderr, "End of file. Something is wrong.(filesystem?)\n"); 
			exit(30);	
		}

		//decrypt block

		memset(ivec,0,BLOCK_SIZE); // reset ivec !!! 
		AES_cbc_encrypt(extrfrom.block,extrfrom.block,BLOCK_SIZE, &randgen,ivec,AES_DECRYPT);
		memcpy(&temp2,extrfrom.block,4);
	
	
		if ( temp2 == extrfrom.id ) {  // if we have found another our block
	
			j = 0;
			if ( outputsize < (BLOCK_SIZE-4) ) { 	
				if ( (status = write(out_file, extrfrom.block+4, outputsize)) == -1 ) {
					errormsg = strerror(errno);
					fprintf(stderr, "write() error in out_file: %s.\n",errormsg);
					exit(30);
				}
				
				close(out_file);	
				printf("Done.\n");
				exit(0);
			} else
			{
				if ( (status = write(out_file, extrfrom.block+4, BLOCK_SIZE-4)) == -1 ) {
					errormsg = strerror(errno);
					fprintf(stderr, "write() error in out_file: %s.\n",errormsg);
					exit(30);
				}
			}
		
			// we have just read BLOCK_SIZE-(length of "id") bytes of file
			outputsize-=BLOCK_SIZE-4;
		
			// generate next decryption key	
			AES_encrypt(extrfrom.keygen,extrfrom.keygen, &extrfrom.key);
			AES_set_decrypt_key(extrfrom.keygen,256,&randgen);
			AES_set_encrypt_key(extrfrom.keygen,256,&extrfrom.key);

			temp2 = 0;
		} else {
			
		       	temp2 = 1; 
		}
		
		// generate next index
		
		AES_encrypt(extrfrom.keygen,extrfrom.keygen, &extrfrom.key);

		AES_set_encrypt_key(extrfrom.keygen,256,&extrfrom.key);
				
	}	

 }	
	printf("Haven't found the lord, oh no no no.");
	//close output	
	close(out_file);	
	exit(0);
}
